qx.Class.define("pf.base.Species",
{
    extend : qx.core.Object,
    
    members: {
        id : "",
        _m : 0.0,
        _q : 0,
        _n : 0.0,
        _t : 0.0,
        
        _reference : null,
        
        _m_reduced : 0.0,
        
        _update: function()
        {
            var reference = this._reference;
            if (reference) {
                this._m_reduced = reference._m*this._m/(reference._m + this._m);
            }
        },
        
        setM: function(m)
        {
            if (m > 0.0) {
                this._m = m;
                this._update();
                return true;
            } else {
                return false;
            }
        },
        
        setQ: function(q)
        {
            this._q = q;
            this._update();
        },
        
        setN: function(n)
        {
            if (n >= 0.0) {
                this._n = n;
                this._update();
                return true;
            } else {
                return false;
            }
        },
        
        setT: function(t)
        {
            if (t > 0.0) {
                this._t = t;
                this._update();
                return true;
            } else {
                return false;
            }
        },

        getN: function()
        {
            return this._n;
        },
        
        getQ: function()
        {
            return this._q;
        },
        
        getT: function()
        {
            return this._t;
        },
        
        getM: function()
        {
            return this._m;
        },
        
        getDebyeLength: function()
        {
            return 7.43e2/Math.abs(this._q)*Math.sqrt(this._t/this._n);
        },

        getPlasmaFrequency: function()
        {
            return 2.10e2*Math.abs(this._q)*Math.sqrt(this._n/this._m);
        },

        getInertialLength: function()
        {
            return 3e10/this.getPlasmaFrequency();
        },

        getThermalVelocity: function()
        {
            return 9.79e5*Math.sqrt(this._t/this._m);
        },
        
        getDeBroglieLength: function()
        {
            return 6.44e-10/Math.sqrt(this._m*this._t);
        },
        
        getMinApproachDistance: function()
        {
            return 1.44e-7*this._q*this._q/this._t;
        },
        
        getMeanDistance: function()
        {
            return Math.pow(4*Math.PI/3*this._n, -1/3);
        },
        
        getGyroFrequency: function(B)
        {
            return 1.52e7*Math.abs(this._q)/this._m*B;
        },
        
        getAlfvenVelocity: function(B)
        {
            return 2.18e15*B/Math.sqrt(this._m*this._n);
        },
        
        getGyroRadius: function(B)
        {
            return this.getThermalVelocity()/
                (2*Math.PI*this.getGyroFrequency(B));
        },
        
        getCoupling: function()
        {
            if (this._q == 0 || this._n == 0) {
                return 0;
            } else {
                var r = this.getMeanDistance()/this.getDebyeLength();
                return 1/3*r*r;
            }
        },
        
        getHoltsmarkField: function()
        {
            return 3.751e-7*Math.abs(this._q)*Math.pow(this._n, 2/3);
        }
        
    },
    
    construct: function(id, m, q, n, t, reference)
    {
        if (m <= 0.0 || n < 0.0 || t <= 0.0) {
            return null;
        }
         
        this.id = id;
        this._m  = m;
        this._q  = q;
        this._n  = n;
        this._t  = t;
        
        this._reference = reference;
        
        this._update();
    }
    
});
